/*
(c) Copyright 2008 Pacito.Sys hppacito@gmail.com

┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘

*/

/* This file is the high level interface to the VGA driver

 A 320x480 1bpp VGA driver (with 640x480 timings)

 */
 
#include "vga_refresh.h"
#include "vga_drv_cmd.h"
#include "vga_drv_defs.h"

unsigned char video_buffer[YLINES*XBYTES];

void c_vga_clrscr( void )
{
int x;

    for (x = 0; x < YLINES*(XBYTES); x++)
        video_buffer[x] = 0;
}

void c_vga_pixel(unsigned int x, unsigned int y, unsigned int color)
{
int v, ptr;
    if (x > (XPIXELS-1) || y > (YLINES-1))
	return;
    ptr = y*(XBYTES)+(x>>XSHIFT);
    v = video_buffer[ptr];
    if (x & 1)
        video_buffer[ptr] = (v & 0xf0) | (color&0x0f);
    else
        video_buffer[ptr] = (v & 0x0f) | (color<<4);
}

// Taken from www.edepot.com/linebenchmark.html

void c_vga_line(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color)
{
int x, y;
int dx, dy;
int incx, incy;
int balance;


	if (x2 >= x1)
	{
		dx = x2 - x1;
		incx = 1;
	}
	else
	{
		dx = x1 - x2;
		incx = -1;
	}

	if (y2 >= y1)
	{
		dy = y2 - y1;
		incy = 1;
	}
	else
	{
		dy = y1 - y2;
		incy = -1;
	}

	x = x1;
	y = y1;

	if (dx >= dy)
	{
		dy <<= 1;
		balance = dy - dx;
		dx <<= 1;

		while (x != x2)
		{
			c_vga_pixel(x, y, color);
			if (balance >= 0)
			{
				y += incy;
				balance -= dx;
			}
			balance += dy;
			x += incx;
		} c_vga_pixel(x, y,color);
	}
	else
	{
		dx <<= 1;
		balance = dx - dy;
		dy <<= 1;

		while (y != y2)
		{
			c_vga_pixel(x, y, color);
			if (balance >= 0)
			{
				x += incx;
				balance -= dy;
			}
			balance += dx;
			y += incy;
		} c_vga_pixel(x, y, color);
	}
}

/* Wrapper to call the assembly VGA driver
   The pointer is to the bitmap buffer
 */ 
void c_vga_refresh( void )
{
    vga_refresh(&video_buffer[0]);
    // it never returns
}

void c_vga_draw(unsigned int cmd, unsigned int args[])
{

    switch (cmd)
    {
	case DCMD_CLRSCR:
	    c_vga_clrscr();
	    break;
	case DCMD_PIXEL:
	    c_vga_pixel(args[0], args[1], args[2]);
	    break;
	case DCMD_LINE:
	    c_vga_line(args[0], args[1], args[2], args[3], args[4]);
	    break;
    }
}

